<!doctype html>
<?php
  date_default_timezone_set("America/New_York");
?>
<html lang="en">
  <head>
    <title>How to work with prepared statements</title>
    <link href="https://jimgerland.com/includes/css/styles.css" rel="stylesheet" />
    <link href="https://jimgerland.com/includes/css/codestyles.css" rel="stylesheet" />   
    <style>
      table, th, td {
        font-family: Verdana, Arial, Sans-Serif;
        font-size: large;
      }
    </style>
  </head>
  <body>
    <h1>How to work with prepared statements</h1>
    <h2>Some methods of the PDO class</h2>
    <div style="font-family: Verdana, Arial, Sans-Serif; font-size: large;">
      <table>
        <thead>
          <tr>
            <th>Method</th>
            <th>Description</th>
          </tr>
        </thead>
        <tbody>
          <tr>
            <td>prepare($sql_statement)</td>
            <td>Prepares the specified SQL statement for execution and returns a PDOStatement object. The specified statement can contain zero or more named (:name) or question mark (?) parameters.</td>
          </tr>
          <tr>
            <td>lastInsertId()</td>
            <td>After an INSERT statement has been executed, this method gets the ID that was automatically generated by MySQL for the row.</td>
          </tr>
        </tbody>
      </table>
    </div>
    <h2>Some methods of the PDOStatement class</h2>
    <div style="font-family: Verdana, Arial, Sans-Serif; font-size: large;">
      <table>
        <thead>
          <tr>
            <th>Method</th>
            <th>Description</th>
          </tr>
        </thead> 
        <tbody>
          <tr>
            <td>bindValue($param, $value)</td>
            <td>Returns TRUE for success and FALSE for failure.</td>
          </tr>
          <tr>
            <td>execute()</td>
            <td>Executes the prepared statement. Returns TRUE for success and FALSE for failure.</td>
          </tr>
          <tr>
            <td>fetchAll()</td>
            <td>Returns an array for all of the rows in the result set.</td>
          </tr>
          <tr>
            <td>fetch()</td>
            <td>Returns an array for the next row in the result set.</td>
          </tr>
          <tr>
            <td>rowCount()</td>
            <td>Returns the number of rows affected by the last statement.</td>
          </tr>
          <tr>
            <td>closeCursor()</td>
            <td>Closes the cursor and frees the connection to the server so other SQL statements may be issued.</td>
          </tr>
        </tbody>
      </table>
    </div>
    <h2>How to use the <code>fetchAll()</code> method to return a result set</h2>
    <div style="font-family: Courier, Arial, Sans-Serif; font-size: large;">
      <pre>
      $query = 'SELECT * FROM products'; 
      $statement = $db->prepare($query);
      $statement->execute(); 
      $products = $statement->fetchAll(); 
      $statement->closeCursor(); 
      foreach ($products as $product) { 
        echo($product['productName'] . "&lt;br /&gt;\n");
      } 
      </pre>
    </div>
    <h2>How to use the fetch() method to loop through a result set</h2>
    <div style="font-family: Courier, Arial, Sans-Serif; font-size: large;">
      <pre>
      $query = 'SELECT * FROM products'; 
      $statement = $db->prepare($query); 
      $statement->execute(); 
      $product = $statement->fetch(); 
      while ($product != null) { 
        echo($product['productName'] . "&lt;br /&gt;\n");
        $product = $statement->fetch(); 
      } 
      $statement->closeCursor();
      </pre>
    </div>
    <h2>How to use named parameters</h2>
    <div style="font-family: Courier, Arial, Sans-Serif; font-size: large;">
      <pre>
      $query = 'SELECT * FROM products 
                WHERE categoryID = :category_id AND listPrice > :price';
      $statement = $db->prepare($query);
      $statement->bindValue(':category_id', <span style="background-color: yellow;">$category_id</span>); 
      $statement->bindValue(':price', <span style="background-color: yellow;">$price</span>); 
      $statement->execute(); 
      $products = $statement->fetchAll(); 
      $statement->closeCursor();
      </pre>
    </div>
    <h2>How to use question mark parameters</h2>
    <div style="font-family: Courier, Arial, Sans-Serif; font-size: large;">
      <pre>
       $query = 'SELECT * FROM products WHERE categoryID = <span style="background-color: yellow;">?</span> AND listPrice > <span style="background-color: yellow;">?</span>'; 
       $statement = $db->prepare($query); 
       $statement->bindValue(1, $category_id); 
       $statement->bindValue(2, $price); 
       $statement->execute(); 
       $products = $statement->fetchAll(); 
       $statement->closeCursor();
      </pre>
    </div>
    <h2>How to modify data</h2>
    <div style="font-family: Courier, Arial, Sans-Serif; font-size: large;">
      <pre>
      // Sample data 
      $category_id = 2; 
      $code = 'hofner'; 
      $name = 'Hofner Icon'; 
      $price = '499.99'; 
       
      // Prepare and execute the statement 
      $query = 'INSERT INTO products 
                VALUES (categoryID, productCode, productName, listPrice) 
                (:category_id, :code, :name, :price)';
      $statement = $db->prepare($query); 
      $statement->bindValue(':category_id', $category_id); 
      $statement->bindValue(':code', $code); 
      $statement->bindValue(':name', $name); 
      $statement->bindValue(':price', $price); 
      $success = $statement->execute(); 
      $row_count = $statement->rowCount(); 
      $statement->closeCursor(); 
       
      // Get the last product ID that was automatically generated
      $product_id = $db->lastInsertId(); 
       
      // Display a message to the user 
      if ($success) { 
        echo("&lt;p&gt;$row_count row(s) was inserted with this ID: $product_id&lt;/p&gt;\n");
      } else {
        echo("&lt;p&gt;No rows were inserted.&lt;/p&gt;\n");
      }
      </pre>
    </div>
    <p><a href=".">Back</a></p>
  </body>
</html>